import java.awt.*;
import javax.swing.*;
import java.awt.geom.*;
import java.util.*;

/**
 * Graph component for displaying chemical signature.
 * 
 * Receives a GraphDataSet object and uses it to draw the graph.
 * 
 * GraphDataSet contains:
 *  * Value for origin (usually 0) * Scale for Y (usually 0%-100%) * Scale for X
 * (minumum atomic weight - maximum atomic weight) * ArrayList of Point objects,
 * denoting where to draw a line and how big to make it.
 * 
 * @author Team NAH
 * @version Spring 2006
 */
public class ChemGraph extends JComponent {

	private GraphDataSet dataSet;

	private Graphics2D g2d;

	private int height, width;

	private final static int maxCharHeight = 20;

	private final static int minFontSize = 8;

	/**
	 * Instanciate a new ChemGraph.
	 * 
	 * Does exactly what it says on the tin.
	 * 
	 * @param GraphDataSet
	 *            A graph dataset object.
	 * @param int
	 *            height of the object.
	 * @param int
	 *            width of the object.
	 */
	public ChemGraph(GraphDataSet dataSet, int height, int width) {
		super();
		this.dataSet = dataSet;
		this.height = height;
		this.width = width;
		setPreferredSize(new Dimension(width, height));
		setVisible(true);
	}

	/**
	 * Set the graph dataset object and redraws.
	 * 
	 * @param GraphDataSet
	 *            A graph dataset object
	 */
	public void setData(GraphDataSet dataSet) {
		this.dataSet = dataSet;
		drawGraph();
		repaint();
	}

	/*
	 * Extract info from the graph data set and draw it.
	 */
	private void drawGraph() {
		Iterator it = dataSet.getPoints().iterator();
		while (it.hasNext()) {
			Point point = (Point) it.next();
			int ax = Math.round(width / 11) * point.getXPoint();
			int ay = 0;
			if (point.getHeight() != 100) {
				ay = (height - 40) - (((height - 60) / 100) * point.getHeight());
			} else {
				ay = 40;
			}
			int bx = Math.round(width / 11) * point.getXPoint();
			int by = height - 40;
			g2d.setColor(Color.blue);
			g2d.setStroke(new BasicStroke(1.0f));
			g2d.draw(new Line2D.Double(ax, ay, bx, by));
			
			//add value to top of point
			g2d.drawString(new Integer(point.getHeight()).toString().concat("%"),ax,ay);
		}

		writeLabels(dataSet.getxLabels(), dataSet.getyLabels(), dataSet
				.getOrigin());
	}

	/**
	 * Paint the graph.
	 * 
	 * Puts the graph together using the Graphics2D library when the object is
	 * instanciated for the first time.
	 * 
	 * @param Graphics
	 *            Graphics object representing the component
	 */
	public void paintComponent(Graphics g) {

		super.paintComponent(g);
		g2d = (Graphics2D) g;

		Dimension d = getSize();
		int gridWidth = d.width / 6;
		int gridHeight = d.height / 2;

		fontMetrics = pickFont(g2d, "", gridWidth);

		// draw the base graph
		g2d.setPaint(Color.white);
		g2d.fill(new Rectangle2D.Double(0, 0,(width - 20), (height - 20)));
		g2d.setColor(Color.black);
		// draw a border around the edge
		g2d.drawRect(0, 0, (width - 20), (height - 20));
		g2d.setColor(Color.darkGray);
		g2d.setStroke(new BasicStroke(3.0f));
		// draw x axis
		g2d.draw(new Line2D.Double(20, (height - 40), (width - 40),
				(height - 40)));
		// draw y axis
		g2d.draw(new Line2D.Double(20, 40, 20, (height - 40)));

		// add the lines
		drawGraph();
	}

	private void writeLabels(String xlabels[], String ylabels[], String origin) {
		final int XLENGTH = 11;
		final int YLENGTH = 11;
		g2d.setColor(Color.black);
		// set the origin value
		g2d.drawString(origin, 12, (height - 26));

		// fill the x axis
		for (int x = 0; x < XLENGTH; x++) {
			g2d.drawString(xlabels[x], (Math.round((width / XLENGTH) * x)),
					(height - 25));
		}
		// fill the y axis
		for (int y = 0; y < YLENGTH; y++) {
			g2d.drawString(ylabels[y], 4, (Math.round(((height - 20) / YLENGTH)
					* (YLENGTH - y))));
		}
	}

	private Dimension totalSize;

	private FontMetrics fontMetrics;

	// utility method to fix the font sizes
	private FontMetrics pickFont(Graphics2D g2d, String longString, int xSpace) {
		boolean fontFits = false;
		Font font = g2d.getFont();
		FontMetrics fontMetrics = g2d.getFontMetrics();
		int size = font.getSize();
		String name = font.getName();
		int style = font.getStyle();

		while (!fontFits) {
			if ((fontMetrics.getHeight() <= maxCharHeight)
					&& (fontMetrics.stringWidth(longString) <= xSpace)) {
				fontFits = true;
			} else {
				if (size <= minFontSize) {
					fontFits = true;
				} else {
					g2d.setFont(font = new Font(name, style, --size));
					fontMetrics = g2d.getFontMetrics();
				}
			}
		}
		return fontMetrics;
	}
}
